@@ -3,10 +3,10 @@ test_that("normboot.flexsurvreg",{
     set.seed(1); b1 <- normboot.flexsurvreg(fite, B=10, newdata=list(age=0))
     set.seed(1); b1 <- normboot.flexsurvreg(fite, B=10, newdata=list(age=50))
     set.seed(1); b2 <- normboot.flexsurvreg(fite, B=10, X=matrix(50,nrow=1))
-    expect_equal(b1, b2)
+    expect_equivalent(b1, b2)
     set.seed(1); b1 <- normboot.flexsurvreg(fite, B=10, newdata=list(age=c(0,50)))
     set.seed(1); b2 <- normboot.flexsurvreg(fite, B=10, X=matrix(c(0,50),nrow=2))
-    expect_equal(b1, b2)   
+    expect_equivalent(b1, b2)   
 
     ## return cov effs, not adjusted
     set.seed(1)
